@@ -1,8 +1,8 @@
 # $File: //member/autrijus/PAR/lib/PAR.pm $ $Author: autrijus $
-# $Revision: #45 $ $Change: 9459 $ $DateTime: 2003/12/28 01:18:54 $ vim: expandtab shiftwidth=4
+# $Revision: #48 $ $Change: 9581 $ $DateTime: 2004/01/03 16:18:29 $ vim: expandtab shiftwidth=4
 
 package PAR;
-$PAR::VERSION = '0.76_99';
+$PAR::VERSION = '0.77_98';
 
 use 5.006;
 use strict;
@@ -15,7 +15,7 @@ PAR - Perl Archive Toolkit
 
 =head1 VERSION
 
-This document describes version 0.76_99 of PAR, released December 28, 2003.
+This document describes version 0.77_98 of PAR, released January 4, 2004.
 
 =head1 SYNOPSIS
 
@@ -383,7 +383,10 @@ sub _tmpfile {
     }
 
     require File::Spec;
-    my $filename = File::Spec->catfile( File::Spec->tmpdir, $_[0] );
+    my $filename = File::Spec->catfile(
+        $ENV{PAR_TEMP} || File::Spec->tmpdir,
+        $_[0],
+    );
     if (-r $filename) {
         open my $fh, '<', $filename or die $!;
         binmode($fh);
@@ -436,7 +439,7 @@ Please submit bug reports to E<lt>bug-par@rt.cpan.orgE<gt>.
 
 =head1 COPYRIGHT
 
-Copyright 2002, 2003 by Autrijus Tang E<lt>autrijus@autrijus.orgE<gt>.
+Copyright 2002, 2003, 2004 by Autrijus Tang E<lt>autrijus@autrijus.orgE<gt>.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
